package movieorganizer.userinterface;

import java.awt.Component;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.LayoutStyle;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;

import javax.swing.RowFilter.ComparisonType;
import javax.swing.border.EmptyBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;


/**
 * The class for the main window with the menu and which in default mode shows
 * the movie titles etc.
 */
public class MainWindow {

    /** Our window where all other elements get attached to. */
    private JFrame frame;
    /** Our movieTable: all movies get listed here. */
    private JTable movieTable;
    /** The TableModel tells the table which data to show. */
    private MovieTableModel tableModel;
    /** We want scroll bars for our table. */
    private JScrollPane scrollPane;

    /** We want a menu bar including a menu and some buttons. */
    private JMenuBar menuBar;
    /** The menu from the menu bar. */
    private JMenu menu;
    /** The menu item for "Add Folder ...". */
    private JMenuItem menuItem;
    /** The file chooser object needed to choose a folder. */
    private JFileChooser folderChooser;

    /** The pop up menu which opens when right mouse-button is pressed. */
    private JPopupMenu popUpMenu;

    /**
     * Creates the main window with the menubar, the filter fields and the
     * table with the movies.
     */
    public MainWindow() {
        frame = new JFrame("MovieOrganizer");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        createSortableTable();
        createMenuBar();

        // At the first start we show a dialog to choose a folder with movies.
        if (MovieManager.getMovies().size() == 0) {
            handleFolderChoosing();
        }

        // Add our table and menu to the splitpane.
        JPanel filterPanel = new JPanel();
        GroupLayout groupLayout = new GroupLayout(filterPanel);
        // Give the filter panel a group layout.
        filterPanel.setLayout(groupLayout);
        setFilterPanelLayout(filterPanel, groupLayout);
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                filterPanel, scrollPane);
        splitPane.setOneTouchExpandable(true);
        splitPane.setDividerLocation(Settings.DIVIDER_LOCATION);
        splitPane.setResizeWeight(0.0);

        // Add the splitpane to the frame.
        scrollPane.setMinimumSize(Settings.TABLEPANE_SIZE);
        frame.getContentPane().add(splitPane);
        frame.setJMenuBar(menuBar);
        frame.setSize(Settings.MAINWINDOW_SIZE);
        frame.setMinimumSize(Settings.MAINWINDOW_SIZE);
        frame.setVisible(true);
    }

    /** Create the search fields with a group layout.
     * @param filterPanel the panel with the filter-elements.
     * @param groupLayout the group-layout from the filterPanel. */
    private void setFilterPanelLayout(final JPanel filterPanel,
            final GroupLayout groupLayout) {
        groupLayout.setAutoCreateGaps(true);

        // Generate the GUI filter elements.
        Font font = new Font(Settings.TEXT_FONT, Font.PLAIN,
                Settings.TEXT_SIZE);
        JLabel filterTitleLabel = new JLabel("Title:");
        filterTitleLabel.setFont(font);
        final JTextField filterTitleText = new JTextField("");
        filterTitleText.setFont(font);

        JLabel filterRatingLabel = new JLabel("Rating >=");
        filterRatingLabel.setFont(font);
        final JTextField filterRatingText = new JTextField("");
        filterRatingText.setFont(font);

        JLabel filterYearLabel = new JLabel("Year >=");
        filterYearLabel.setFont(font);
        final JTextField filterYearText = new JTextField("");
        filterYearText.setFont(font);

        JLabel filterDurationLabel = new JLabel("Duration >=");
        filterDurationLabel.setFont(font);
        final JTextField filterDurationText = new JTextField("");
        filterDurationText.setFont(font);

        JLabel filterDirectorLabel = new JLabel("Director:");
        filterDirectorLabel.setFont(font);
        final JTextField filterDirectorText = new JTextField("");
        filterDirectorText.setFont(font);

        JLabel filterGenresLabel = new JLabel("Genres:");
        DefaultListModel defModel = new DefaultListModel();
        for (int i = 0; i < Settings.ALL_GENRES.length; i++) {
            defModel.addElement(Settings.ALL_GENRES[i]);
        }

        final JList genresList = new JList(defModel);
        genresList.setFont(font);
        final JScrollPane filterGenresList = new JScrollPane(genresList);

        // Add the key events for all text fields.
        KeyAdapter keyAdapter = new KeyAdapter() {
            @Override
            public void keyReleased(final KeyEvent e) {
                filterCompleteTable(filterTitleText, filterDirectorText,
                        filterRatingText, filterDurationText, filterYearText,
                        genresList);
            }
        };
        filterTitleText.addKeyListener(keyAdapter);
        filterRatingText.addKeyListener(keyAdapter);
        filterYearText.addKeyListener(keyAdapter);
        filterDurationText.addKeyListener(keyAdapter);
        filterDirectorText.addKeyListener(keyAdapter);
        filterGenresLabel.addKeyListener(keyAdapter);
        genresList.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(final MouseEvent e) {
                filterCompleteTable(filterTitleText, filterDirectorText,
                        filterRatingText, filterDurationText, filterYearText,
                        genresList);
            }
        });

        JButton filterResetB = new JButton("Clear");
        filterResetB.setFont(new Font(Settings.TEXT_FONT, Font.PLAIN,
                Settings.TEXT_SIZE));
        filterResetB.addActionListener(new ActionListener() {
            @SuppressWarnings("unchecked")
            @Override
            public void actionPerformed(final ActionEvent arg0) {
                TableRowSorter<MovieTableModel> rowSorter =
                    (TableRowSorter<MovieTableModel>) movieTable.getRowSorter();
                rowSorter.setRowFilter(null);
                filterTitleText.setText("");
                filterDirectorText.setText("");
                filterRatingText.setText("");
                filterDurationText.setText("");
                genresList.clearSelection();
                filterYearText.setText("");

            }
        });

        groupLayout
                .setHorizontalGroup(groupLayout
                        .createParallelGroup(GroupLayout.Alignment.LEADING)
                        .addGroup(
                                groupLayout
                                   .createSequentialGroup()
                                   .addContainerGap()
                                   .addGroup(
                                           groupLayout
                                           .createParallelGroup(
                                              GroupLayout.Alignment.LEADING)
                                           .addComponent(
                                              filterTitleLabel)
                                           .addComponent(
                                              filterDirectorLabel))
                                   .addPreferredGap(
                                         LayoutStyle.ComponentPlacement.RELATED)
                                   .addGroup(
                                                groupLayout
                                                .createParallelGroup(
                                                  GroupLayout.Alignment.LEADING,
                                                                false)
                                                  .addComponent(
                                                       filterTitleText)
                                                  .addComponent(
                                                       filterDirectorText,
                                                       GroupLayout.DEFAULT_SIZE,
                                                       120,
                                                       Short.MAX_VALUE))
                                    .addPreferredGap(
                                      LayoutStyle.ComponentPlacement.UNRELATED)
                                    .addGroup(
                                       groupLayout
                                         .createParallelGroup(
                                            GroupLayout.Alignment.LEADING)
                                         .addComponent(
                                            filterDurationLabel)
                                         .addComponent(
                                            filterRatingLabel))
                                    .addPreferredGap(
                                       LayoutStyle.ComponentPlacement.RELATED)
                                    .addGroup(
                                       groupLayout
                                        .createParallelGroup(
                                          GroupLayout.Alignment.LEADING,
                                                                false)
                                        .addComponent(
                                          filterDurationText)
                                        .addComponent(
                                          filterRatingText,
                                          GroupLayout.DEFAULT_SIZE,
                                                    50, Short.MAX_VALUE))
                                     .addGap(18, 18, 18)
                                        .addComponent(filterGenresList,
                                                GroupLayout.PREFERRED_SIZE,
                                                100, GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addGroup(
                                            groupLayout
                                              .createParallelGroup(
                                                GroupLayout.Alignment.LEADING)
                                              .addGroup(
                                                groupLayout
                                                 .createSequentialGroup()
                                                   .addComponent(
                                                      filterYearLabel)
                                                   .addPreferredGap(
                                         LayoutStyle.ComponentPlacement.RELATED,
                                         20, 20)
                                         .addComponent(
                                            filterYearText,
                                            GroupLayout.PREFERRED_SIZE, 76,
                                            GroupLayout.PREFERRED_SIZE))
                                         .addComponent(
                                            filterResetB,
                                            GroupLayout.PREFERRED_SIZE, 138,
                                            GroupLayout.PREFERRED_SIZE))
                                        .addGap(2430, 2430, 2430)));
        groupLayout
          .setVerticalGroup(groupLayout
            .createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(
                groupLayout
                  .createSequentialGroup()
                  .addContainerGap()
                  .addGroup(
                     groupLayout
                        .createParallelGroup(
                           GroupLayout.Alignment.TRAILING,
                           false)
                        .addGroup(
                           GroupLayout.Alignment.LEADING,
                           groupLayout
                             .createSequentialGroup()
                             .addGroup(
                                groupLayout
                                  .createParallelGroup(
                                     GroupLayout.Alignment.LEADING)
                                  .addComponent(
                                     filterYearText,
                                     GroupLayout.PREFERRED_SIZE,
                                     GroupLayout.DEFAULT_SIZE,
                                     GroupLayout.PREFERRED_SIZE)
                                  .addComponent(
                                     filterYearLabel))
                                  .addPreferredGap(
                                     LayoutStyle.ComponentPlacement.RELATED,
                                       GroupLayout.DEFAULT_SIZE,
                                       Short.MAX_VALUE)
                                  .addComponent(
                                      filterResetB))
                                  .addGroup(
                                     GroupLayout.Alignment.LEADING,
                                       groupLayout
                                     .createSequentialGroup()
                                     .addGroup(
                                        groupLayout
                                          .createParallelGroup(
                                             GroupLayout.Alignment.BASELINE)
                                          .addComponent(
                                             filterTitleLabel)
                                          .addComponent(
                                             filterDurationLabel)
                                          .addComponent(
                                             filterDurationText,
                                             GroupLayout.PREFERRED_SIZE,
                                             GroupLayout.DEFAULT_SIZE,
                                             GroupLayout.PREFERRED_SIZE)
                                          .addComponent(
                                             filterTitleText,
                                             GroupLayout.PREFERRED_SIZE,
                                             GroupLayout.DEFAULT_SIZE,
                                             GroupLayout.PREFERRED_SIZE))
                                          .addGap(
                                             18, 18, 18)
                                          .addGroup(
                                             groupLayout
                                          .createParallelGroup(
                                             GroupLayout.Alignment.BASELINE)
                                          .addComponent(
                                             filterDirectorLabel)
                                          .addComponent(
                                             filterRatingText,
                                             GroupLayout.PREFERRED_SIZE,
                                             GroupLayout.DEFAULT_SIZE,
                                             GroupLayout.PREFERRED_SIZE)
                                          .addComponent(
                                             filterRatingLabel)
                                          .addComponent(
                                             filterDirectorText,
                                             GroupLayout.PREFERRED_SIZE,
                                             GroupLayout.DEFAULT_SIZE,
                                             GroupLayout.PREFERRED_SIZE)))
                                          .addComponent(
                                             filterGenresList,
                                             GroupLayout.Alignment.LEADING,
                                             0, 0, Short.MAX_VALUE))
                                        .addGap(10, 10, 10)));
    }

    /**
     * Filters the movieTable for title, director, genres, rating, duration and
     * year.
     * @param filterDirectorText the textfield for filtering the director.
     * @param filterTitleText the textfield for filtering the title.
     * @param filterRatingText the textfield for filtering the rating.
     * @param filterDurationText the textfield for filtering the duration.
     * @param filterYearText the textfield for filtering the year.
     * @param filterGenresList the list for filtering the genres.
     */
    @SuppressWarnings("unchecked")
    private void filterCompleteTable(final JTextField filterTitleText,
            final JTextField filterDirectorText,
            final JTextField filterRatingText,
            final JTextField filterDurationText,
            final JTextField filterYearText,
            final JList filterGenresList) {
        ArrayList<RowFilter<MovieTableModel, Object>> filters =
            new ArrayList<RowFilter<MovieTableModel, Object>>();

        String title = filterTitleText.getText();
        String director = filterDirectorText.getText();

        float rating = XMLHelper.toFloat(filterRatingText.getText());
        int duration = XMLHelper.toInt(filterDurationText.getText());
        int year = XMLHelper.toInt(filterYearText.getText());

        // Filter the titles.
        if (title != null && title.length() > 0) {
            filters.add(filterStringColumn(MovieTableModel.COL_TITLE, title));
        }

        // Filter the directors.
        if (director != null && director.length() > 0) {
            filters.add(filterStringColumn(MovieTableModel.COL_DIRECTOR,
                    director));
        }

        // Filter the genres.
        int[] selectedGenres = filterGenresList.getSelectedIndices();
        for (int i = 0; i < selectedGenres.length; i++) {
            String genre = Settings.ALL_GENRES[selectedGenres[i]];
            filters.add(filterStringColumn(MovieTableModel.COL_GENRES, genre));
        }

        // Filter the ratings.
        if (rating >= 0) {
            filters.add(filterNumberColumn(MovieTableModel.COL_RATING, rating));
        }

        // Filter the durations.
        if (duration >= 0) {
            filters.add(filterNumberColumn(MovieTableModel.COL_DURATION,
                    duration));
        }

        // Filter for the year.
        if (year >= 0) {
            filters.add(filterNumberColumn(MovieTableModel.COL_YEAR, year));
        }

        if (filters.size() > 0) { // If we have filters, apply them.
            TableRowSorter<MovieTableModel> rowSorter =
                (TableRowSorter<MovieTableModel>) movieTable.getRowSorter();
            rowSorter.setRowFilter(RowFilter.andFilter(filters));
        } else {
            TableRowSorter<MovieTableModel> rowSorter =
                (TableRowSorter<MovieTableModel>) movieTable.getRowSorter();
            rowSorter.setRowFilter(null); // Show all entries.
        }
    }

    /** This class gives the cells of JTable a larger font and cellpadding. */
    private class LargerFontCellRenderer extends DefaultTableCellRenderer {
        /** The standard serial-version-UID. */
        private static final long serialVersionUID = 1L;

        /** The cell-renderer for the normal cells (not the header).
         * @param table the table.
         * @param value the value.
         * @param isSelected true/false.
         * @param hasFocus true/false.
         * @param row the row.
         * @param column the column.
         * @return the new cell-renderer. */
        public Component getTableCellRendererComponent(final JTable table,
                final Object value, final boolean isSelected,
                final boolean hasFocus, final int row, final int column) {
            JLabel c = (JLabel) super.getTableCellRendererComponent(table,
                    value, isSelected, hasFocus, row, column);
            c.setFont(new Font(Settings.TEXT_FONT, Font.PLAIN,
                    Settings.TEXT_SIZE));
            c.setVerticalAlignment(JLabel.CENTER);
            c.setHorizontalAlignment(JLabel.CENTER);
            int cellpadding = Settings.TABLE_CELLPADDING;
            c.setBorder(BorderFactory.createCompoundBorder(c.getBorder(),
                    new EmptyBorder(cellpadding, cellpadding, cellpadding,
                            cellpadding)));
            return c;
        }
    }

    /** The class gives the header of a JTable a larger font and cellpadding. */
    private class LargerFontHeaderRenderer extends
            TableHeaderRenderer {
        /** The default serial-version-UID. */
        private static final long serialVersionUID = 1L;

        /** The cell-renderer for the normal cells (not the header).
         * @param table the table.
         * @param value the value.
         * @param isSelected true/false.
         * @param hasFocus true/false.
         * @param row the row.
         * @param column the column.
         * @return the new cell-renderer. */
        public Component getTableCellRendererComponent(final JTable table,
                final Object value, final boolean isSelected,
                final boolean hasFocus, final int row, final int column) {
            super.getTableCellRendererComponent(table, value, isSelected,
                    hasFocus, row, column);
            JLabel c = (JLabel) super.getTableCellRendererComponent(table,
                    value, isSelected, hasFocus, row, column);
            c.setFont(new Font(Settings.TEXT_FONT, Font.BOLD,
                    Settings.TEXT_SIZE));
            c.setVerticalAlignment(JLabel.CENTER);
            c.setHorizontalAlignment(JLabel.CENTER);
            int cellpadding = Settings.TABLE_CELLPADDING;
            c.setBorder(BorderFactory.createCompoundBorder(c.getBorder(),
                    new EmptyBorder(cellpadding, cellpadding, cellpadding,
                            cellpadding)));
            return c;
        }

    }

    /** Create the sortable table which handles click-events. */
    private void createSortableTable() {
        // We only want to allow single line selection.
        tableModel = new MovieTableModel();
        movieTable = new JTable(tableModel);
        movieTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        movieTable.setRowSelectionAllowed(true);

        // Our table should be sortable.
        TableRowSorter<TableModel> rowSorter = new TableRowSorter<TableModel>(
                tableModel);
        movieTable.setRowSorter(rowSorter);

        // If the table gets to big, we want scrollbars.
        scrollPane = new JScrollPane(movieTable);

        // Set the preferred column sizes.
        movieTable.getColumnModel().getColumn(MovieTableModel.COL_TITLE)
                .setMinWidth(Settings.COL_TITLE_MINSIZE);
        movieTable.getColumnModel().getColumn(MovieTableModel.COL_RATING)
                .setMinWidth(Settings.COL_RATING_SIZE);
        movieTable.getColumnModel().getColumn(MovieTableModel.COL_RATING)
                .setMaxWidth(Settings.COL_RATING_SIZE);
        movieTable.getColumnModel().getColumn(MovieTableModel.COL_YEAR)
                .setMinWidth(Settings.COL_YEAR_SIZE);
        movieTable.getColumnModel().getColumn(MovieTableModel.COL_YEAR)
                .setMaxWidth(Settings.COL_YEAR_SIZE);
        movieTable.getColumnModel().getColumn(MovieTableModel.COL_DURATION)
                .setMinWidth(Settings.COL_DURATION_SIZE);
        movieTable.getColumnModel().getColumn(MovieTableModel.COL_DURATION)
                .setMaxWidth(Settings.COL_DURATION_SIZE);
        movieTable.getColumnModel().getColumn(MovieTableModel.COL_DIRECTOR)
                .setMinWidth(Settings.COL_DIRECTOR_MINSIZE);
        movieTable.getColumnModel().getColumn(MovieTableModel.COL_GENRES)
                .setMinWidth(Settings.COL_GENRES_MINSIZE);

        // Give the columns a larger font, cellpadding and higher rows.
        LargerFontCellRenderer cellRenderer = new LargerFontCellRenderer();
        LargerFontHeaderRenderer headerRenderer =
            new LargerFontHeaderRenderer();
        for (int i = 0; i < movieTable.getColumnCount(); i++) {
            movieTable.getColumnModel().getColumn(i).setCellRenderer(
                    cellRenderer);
            movieTable.getColumnModel().getColumn(i).setHeaderRenderer(
                    headerRenderer);
        }
        movieTable.getTableHeader().setPreferredSize(new Dimension(
                movieTable.getTableHeader().getWidth(),
                Settings.TABLE_ROWHEIGHT));
        movieTable.setRowHeight(Settings.TABLE_ROWHEIGHT);

        // Create a PopUpMenu when the right button is pressed.
        popUpMenu = new JPopupMenu();
        createRightClickPopUpMenu();

        /*
         * Our table needs to respond to two events: doubleclicks and pressing
         * the "enter" key. In both cases a popup should be opened.
         */
        TableListener tableListener = new TableListener(movieTable, tableModel);
        movieTable.addMouseListener(tableListener);
        movieTable.addKeyListener(tableListener);
    }

    /** Returns a filter for the given column for the given expression.
     * @param col the column.
     * @param expression the regular-expression.
     * @return the RowFilter. */
    private RowFilter<MovieTableModel, Object> filterStringColumn(final int col,
            final String expression) {
        RowFilter<MovieTableModel, Object> rf = null;
        try {
            rf = RowFilter.regexFilter("(?i).*(" + expression + ").*", col);
            // rf = RowFilter.regexFilter(Pattern.compile(expression,
            // Pattern.CASE_INSENSITIVE), col);
        } catch (Exception e) {
            System.out.println("Error @ filterStringColumn");
        }
        return rf;
    }

    /**
     * Returns a filter for the given column for all values >= compareValue.
     * @param col the column.
     * @param compareValue the minimum value.
     * @return the RowFilter.
     */
    private RowFilter<MovieTableModel, Object> filterNumberColumn(final int col,
            final Number compareValue) {
        RowFilter<MovieTableModel, Object> rf = null;
        try {
            ArrayList<RowFilter<MovieTableModel, Object>> filters =
                new ArrayList<RowFilter<MovieTableModel, Object>>();
            rf = RowFilter
                    .numberFilter(ComparisonType.AFTER, compareValue, col);
            filters.add(rf);
            rf = RowFilter
                    .numberFilter(ComparisonType.EQUAL, compareValue, col);

            filters.add(rf);
            rf = RowFilter.orFilter(filters);
        } catch (Exception e) {
            System.out.println("Error @ filterNumberColumn");
        }
        return rf;
    }

    /** Create the menu with its components and its action listeners. */
    private void createMenuBar() {
        menuBar = new JMenuBar();
        menu = new JMenu("File");

        // We need to be able to add folders.
        menuItem = new JMenuItem("Add movie folder ...");
        menuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(final ActionEvent e) {
                handleFolderChoosing();
            }
        });
        menu.add(menuItem);
        menuBar.add(menu);

        // We want to be able to remove folders.
        final JMenu subMenu = new JMenu("Remove folder:");
        menu.add(subMenu);

        menuItem.addMouseListener(new MouseListener() {
            @Override
            public void mouseClicked(final MouseEvent arg0) {
            }

            @Override
            public void mouseEntered(final MouseEvent arg0) {
                subMenu.removeAll(); // Remove all folders.
                final String[] movieFolders = MovieManager.getMovieFolders();

                // Add the actual folders.
                for (int i = 0; i < movieFolders.length; i++) {
                    final String folder = movieFolders[i];
                    menuItem = new JMenuItem(folder);
                    menuItem.addActionListener(new ActionListener() {
                        @Override
                        public void actionPerformed(final ActionEvent arg0) {
                            MovieManager.removeFolder(folder);
                            tableModel
                                    .setTableContent(MovieManager.getMovies());
                            tableModel.fireTableDataChanged();
                            MovieManager.writeMoviesAsXml();
                        }
                    });
                    subMenu.add(menuItem);
                }
            }

            @Override
            public void mouseExited(final MouseEvent arg0) {
            }

            @Override
            public void mousePressed(final MouseEvent arg0) {
            }

            @Override
            public void mouseReleased(final MouseEvent arg0) {
            }

        });
    }

    /**
     * Creates the pop up menu which opens when the right mouse button is
     * clicked. It gives the user the option to select an other imdb-entry for
     * the selected movie.
     */
    private void createRightClickPopUpMenu() {
        // One button to change the title of the selected movie.
        JMenuItem mItem = new JMenuItem("Change title to ...");
        mItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(final ActionEvent e) {
                int selectedRow = movieTable.getSelectedRow();
                selectedRow = movieTable.getRowSorter().convertRowIndexToModel(
                        selectedRow);
                String newTitle = (String) JOptionPane.showInputDialog(
                        "Please enter a new title:", tableModel
                                .getMovieFromRow(selectedRow).getTitle());
                if (newTitle != null && newTitle.length() > 0) {
                    // Update the movie title.

                    tableModel.getMovieFromRow(selectedRow).setTitle(newTitle);
                    // Update the table.
                    updateTable(MovieManager.getMovies());
                    // Write the updated title to the XML.
                    MovieManager.writeMoviesAsXml();
                }
            }
        });
        popUpMenu.add(mItem);

        // One button to open the folder of the movie.
        mItem = new JMenuItem("Open folder");
        mItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(final ActionEvent arg0) {
                int selectedRow = movieTable.getSelectedRow();
                selectedRow = movieTable.getRowSorter().convertRowIndexToModel(
                        selectedRow);
                URI folder = tableModel.getMovieFromRow(selectedRow).getURI();
                if (folder != null) {
                    try {
                        Desktop.getDesktop().browse(folder);
                    } catch (IOException e) {
                        System.out.println("Couldn't open folder...");
                    }
                }
            }
        });
        popUpMenu.add(mItem);

        // A sub menu for the different imbd-entries.
        final JMenu subMenu = new JMenu("Change imdb-entry to");
        popUpMenu.add(subMenu);

        // One button to refresh the imdb-information from the server.
        mItem = new JMenuItem("Refresh imdb-data");
        mItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(final ActionEvent arg0) {
                int selectedRow = movieTable.getSelectedRow();
                selectedRow = movieTable.getRowSorter().convertRowIndexToModel(
                        selectedRow);
                tableModel.getMovieFromRow(selectedRow).getMetaDataFromIMDB();
                // Update the table.
                updateTable(MovieManager.getMovies());
                // Write the updated movie to the XML.
                MovieManager.writeMoviesAsXml();
            }
        });
        popUpMenu.add(mItem);

        // One button to delete this the chosen entry from the table.
        mItem = new JMenuItem("Remove entry");
        mItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(final ActionEvent arg0) {
                int selectedRow = movieTable.getSelectedRow();
                selectedRow = movieTable.getRowSorter().convertRowIndexToModel(
                        selectedRow);
                MovieManager.removeMovie(tableModel
                        .getMovieFromRow(selectedRow));
                tableModel.removeRow(selectedRow);
                tableModel.fireTableDataChanged();
            }
        });
        popUpMenu.add(mItem);

        // Handle the click events.
        movieTable.addMouseListener(new TablePopUpListener(movieTable,
                tableModel, popUpMenu, subMenu, this));
    }

    /**
     * Show the dialog to choose a folder and evaluate the choice. This means
     * searching the selected folders and subfolders recursively for movie
     * files. Afterwards the _movieTable gets refreshed to also show the new
     * movies.
     */
    public final void handleFolderChoosing() {
        // We need a dialog to choose the directories.
        folderChooser = new JFileChooser();
        folderChooser.setDialogTitle("MovieOrganizer: "
                + "Please choose a folder with your movies");
        folderChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        int returnVal = folderChooser.showOpenDialog(null);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File chosenFolder = folderChooser.getSelectedFile();
            MovieManager.addFolder(chosenFolder);
            MovieManager.searchFolders();
            MovieManager.writeMoviesAsXml();
            updateTable(MovieManager.getMovies());
        }
    }

    /** Updates the entries in the table.
     * @param movies the new movie HashMap. */
    public final void updateTable(final HashMap<String, Movie> movies) {
        tableModel.setTableContent(movies);
        tableModel.fireTableDataChanged();
    }

}
