package movieorganizer.userinterface;
import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;

/** This class handles reading the movie-filenames etc. */
public final class MovieManager {
    /** Names of folders we search for movies. These should be full paths. */
    private static String[] folderNames;
    /** Valid suffixes for movie files. For now, just take .avi. */
    private static ArrayList<String> movieFileSuffixes;
    /**
     * Movies in our database, indexed by their file name (so that we can tell
     * in constant time whether we already considered a movie with a given file
     * name or not).
     */
    private static HashMap<String, Movie> movies;
    /**
     * Search folder for movie files and add those not already present to
     * movies. TODO(bast): Also delete movies from the databases which are no
     * longer in the folder.
     */

    /** This class should not be instantiated. */
    private MovieManager() {
    }

    /** Search the selected folders for movies with the selected suffixes. */
    public static void searchFolders() {
        if (movies == null) {
            movies = new HashMap<String, Movie>();
        }
        for (String f : folderNames) {
            File folder = new File(f);
            addMovies(folder);
        }
    }

    /**
     * Helper function which gets called from searchFolders(). Needed for
     * recursively searching the folders.
     * @param folder a folder from the HDD.
     */
    private static void addMovies(final File folder) {
        File[] subFilesAndFolders = folder.listFiles();

        for (File f : subFilesAndFolders) {
            if (f.isDirectory()) {
                addMovies(f);
            } else {
                String fileName = f.getName();
                if (hasMovieSuffix(fileName)) {
                    if (!movies.containsKey(fileName)) {
                        Movie movie = new Movie(fileName, f.getAbsolutePath());
                        movie.getMetaDataFromIMDB();
                        movies.put(fileName, movie);
                    }
                }
            }
        }

    }

    /** Checks if a String has one of the movieFileSuffixes.
     * @param s the suffix.
     * @return true/false. */
    private static boolean hasMovieSuffix(final String s) {
        for (int i = 0; i < movieFileSuffixes.size(); i++) {
            if (s.endsWith("." + movieFileSuffixes.get(i))) {
                return true;
            }
        }

        return false;
    }

    /**
     * Load folder names and the valid file suffixes. TODO(mirko): find out all
     * relevant suffixes and add them to movieFileSuffixes.
     */
    public static void initialize() {
        movieFileSuffixes = new ArrayList<String>();
        movieFileSuffixes.add("avi");
        movieFileSuffixes.add("iso");

        folderNames = new String[0];

        movies = new HashMap<String, Movie>();

        readMoviesAsXml();
    }

    /** Get all movies from the selected folders.
     * @return the movies. */
    public static HashMap<String, Movie> getMovies() {
        return movies;
    }

    /** Removes the given movie if its in the HashMap.
     * @param m the movie. */
    public static void removeMovie(final Movie m) {
        if (movies.containsKey(m.getFileName())) {
            movies.remove(m.getFileName());
        }
    }

    /** The user should be able to add folders.
     * @param folder the folder. */
    public static void addFolder(final File folder) {
        String[] newFolderNames = new String[folderNames.length + 1];
        // Copy the old folders.
        for (int i = 0; i < folderNames.length; i++) {
            newFolderNames[i] = folderNames[i];
        }
        // Add the new folder.
        newFolderNames[newFolderNames.length - 1] = folder.getAbsolutePath();

        // Replace the old folder collection by the new folder collection.
        folderNames = newFolderNames;
    }

    /** Write all movies and the complete folders to a specified .XML file. */
    public static void writeMoviesAsXml() {
        try {
            XMLEncoder encoder = new XMLEncoder(new BufferedOutputStream(
                    new FileOutputStream(Settings.INFO_FILE)));

            for (int i = 0; i < folderNames.length; i++) {
                encoder.writeObject(folderNames[i]);
            }

            for (String key : movies.keySet()) {
                encoder.writeObject(movies.get(key));
            }

            encoder.close();
        } catch (Exception e) {
            System.out.println(e);
        }
    }

    /** Read all movies from the specified .XML file INFO_FILE. */
    public static void readMoviesAsXml() {
        movies = new HashMap<String, Movie>();
        try {
            XMLDecoder dec =
                new XMLDecoder(new FileInputStream(Settings.INFO_FILE));
            Object object = null;
            while ((object = dec.readObject()) != null) {
                if (object.getClass() == Movie.class) {
                    Movie m = (Movie) object;
                    movies.put(m.getFileName(), m);
                } else if (object.getClass() == String.class) {
                    try {
                        String folderName = (String) object;
                        File folder = new File(folderName);
                        addFolder(folder);
                    } catch (Exception e) {
                        System.out.println(e);
                    }
                }
            }
            dec.close();
        } catch (ArrayIndexOutOfBoundsException e) {
            // This exception is thrown when we dec.readObject
            // has no more objects to read.
            System.out.println("MovieManager: all objects have been read.");
        } catch (FileNotFoundException e) {
            // If we have no file, then we simply load nothing.
            movies = new HashMap<String, Movie>();
        }
    }

    /** Returns all movie folders which the user added.
     * @return the folders. */
    public static String[] getMovieFolders() {
        return folderNames;
    }

    /**
     * Removes a movie folder (if its available) and all its movies from the
     * HashMap.
     * @param folder the folder.
     */
    public static void removeFolder(final String folder) {
        int index = -1;
        for (int i = 0; i < folderNames.length; i++) {
            if (folderNames[i].equals(folder)) {
                index = i;
            }
        }
        if (index > -1) { // This means we've found the folder.
            String[] newFolderNames = new String[folderNames.length - 1];

            // Remove it from the folderNames array.
            for (int i = 0; i < folderNames.length; i++) {
                // We skip the one folder which we want to remove.
                if (i < index) {
                    newFolderNames[i] = folderNames[i];
                } else if (i > index) {
                    newFolderNames[i - 1] = folderNames[i];
                }
            }
            folderNames = newFolderNames;

            // Remove all movies from this folder from the HashMap.
            ArrayList<String> keysToRemove = new ArrayList<String>();
            for (String key : movies.keySet()) {
                Movie m = movies.get(key);
                if (m.getPath().contains(folder)) {
                    keysToRemove.add(key);
                }
            }
            for (int i = 0; i < keysToRemove.size(); i++) {
                movies.remove(keysToRemove.get(i));
            }
        }
    }

}
